@@ -64,7 +64,8 @@ class Login:
         token_endpoint="https://auth.mozilla.auth0.com/oauth/token",
         web_console=False,
         issuer_domain=None,
-        cache=True
+        cache=True,
+        print_url=False,
     ):
 
         # We use this for tracking various bits
@@ -113,12 +114,18 @@ class Login:
         self.last_state_check = None
         self.max_sleep_no_state_check = 2  # seconds
 
-        # Whether we should open the AWS web console or not
+        # Whether we should open the AWS web console or not and whether we
+        # should print the URL to stdout
+        self.print_url = print_url
         self.web_console = web_console
 
         # If we're using the AWS CLI output, what profile should we use
         self.profile_name = profile_name
 
+        # Whether we should print the output map, which isn't used by
+        # output that is meant to be consumed programmatically
+        self.print_output_map = True
+
         # This used by the web application to poll the login state
         self.state = "pending"
         self.web_state = {
@@ -165,22 +172,22 @@ class Login:
                 if self.role_arn is None:
                     if self.batch:
                         self.exit(
-                            'Unable to fetch AWS STS credentials with ID '
-                            'token. Exiting due to batch mode.')
+                            "Unable to fetch AWS STS credentials with ID "
+                            "token. Exiting due to batch mode.")
                         return False
                     else:
                         print(
-                            'Unable to assume IAM role. Spawning web role '
-                            'picker to pick a different role.',
+                            "Unable to assume IAM role. Spawning web role "
+                            "picker to pick a different role.",
                             file=sys.stderr)
-                if result == 'finished':
+                if result == "finished":
                     return True
 
         if self.token is not None and self.role_arn is None:
             logger.debug(
                 "We have a cached ID token but either no role was passed on "
                 "the command line or it wasn't valid. Show the role picker")
-            self.state = 'redirecting'
+            self.state = "redirecting"
             url_parameters = {
                 "state": self.oidc_state,
                 "code": "this value is unused"
@@ -250,11 +257,11 @@ class Login:
                        URI. For example 'error' and 'error_description'
         :return:
         """
-        if kwargs.get('error'):
+        if kwargs.get("error"):
             self.exit((
                 "Received an error response from the identity provider in "
                 "response to the /authorize request : {}".format(
-                    kwargs.get('error_description'))
+                    kwargs.get("error_description"))
             ))
             return False
 
@@ -315,7 +322,7 @@ class Login:
                 key=self.jwks,
                 audience=self.client_id)
         except JWTError as e:
-            logger.error('ID Token failed validation : {}'.format(e))
+            logger.error("ID Token failed validation : {}".format(e))
             return None
         logger.debug("ID token dict : {}".format(self.id_token_dict))
         return self.id_token_dict
@@ -323,7 +330,7 @@ class Login:
     def get_role_map(self):
         # get the role map, either from cache or from the endpoint
         self.state = "getting_role_map"
-        url = "{}roles".format(self.idtoken_for_roles_url)
+        url = self.idtoken_for_roles_url
         self.role_map = get_roles_and_aliases(
             endpoint=url,
             token=self.token["id_token"],
@@ -336,7 +343,7 @@ class Login:
             return False
 
         logger.debug(
-            'Roles and aliases are {}'.format(self.role_map))
+            "Roles and aliases are {}".format(self.role_map))
         return self.role_map
 
     def exchange_token_for_credentials(self):
@@ -353,20 +360,20 @@ class Login:
             self.print_output()
             return self.state
         except STSWarning as e:
-            if e.args[1] == 'AccessDenied':
+            if len(e.args) > 1 and e.args[1] == "AccessDenied":
                 # Not authorized to perform sts:AssumeRoleWithWebIdentity
                 # Either that role doesn't exist or it exists but doesn't
                 # permit the user because of the conditions
                 # Either way, lets refresh the group role map in case it's out
                 # of date
-                logger.debug('Unable to assume role {}'.format(self.role_arn))
+                logger.debug("Unable to assume role {}".format(self.role_arn))
                 self.cache = False
                 self.get_role_map()
                 if self.batch:
                     self.exit(
                         "Unable to assume role. Shutting down due to batch "
                         "mode being enabled")
-                    return 'error'
+                    return "error"
 
                 self.state = "role_picker"
                 if self.role_arn in self.role_map.get("roles", []):
@@ -375,10 +382,10 @@ class Login:
                 if len(self.role_map.get("roles", [])) <= 1:
                     self.exit(
                         "Sorry, no valid roles available. Shutting down.")
-                    return 'error'
-            elif e.args[1] == 'ExpiredTokenException':
+                    return "error"
+            elif len(e.args) > 1 and e.args[1] == "ExpiredTokenException":
                 logger.debug(
-                    'AWS says that the ID token is expired : {}'.format(e[2]))
+                    "AWS says that the ID token is expired : {}".format(e[2]))
                 self.token = None
                 url_parameters = {
                     "scope": self.oidc_scope,
@@ -392,14 +399,14 @@ class Login:
                 url = "{}?{}".format(self.authorization_endpoint,
                                      urlencode(url_parameters))
                 self.state = "restart_auth"
-                self.web_state['idpUrl'] = url
-                return 'restart_auth'
+                self.web_state["idpUrl"] = url
+                return "restart_auth"
             else:
                 self.exit("Unable to contact AWS : {}".format(e))
-                return 'error'
+                return "error"
         except Exception:
             self.exit("Unable to contact AWS : {}".format("".join(traceback.format_exception(*sys.exc_info()))))
-            return 'error'
+            return "error"
 
     def print_output(self):
         # TODO: Create a global config object?
@@ -414,9 +421,9 @@ class Login:
                      for x in self.credentials
                      if x in ENV_VARIABLE_NAME_MAP})
                 output_map.update({
-                    'AWS_PROFILE': None,
-                    'AWS_SHARED_CREDENTIALS_FILE': None,
-                    'MAWS_PROMPT': self.profile_name})
+                    "AWS_PROFILE": None,
+                    "AWS_SHARED_CREDENTIALS_FILE": None,
+                    "MAWS_PROMPT": self.profile_name})
             elif self.output == "shared":
                 # Write the credentials
                 path = write_aws_shared_credentials(
@@ -424,9 +431,9 @@ class Login:
                     self.credentials)
                 if path:
                     output_map.update({
-                        'AWS_PROFILE': self.profile_name,
-                        'AWS_SHARED_CREDENTIALS_FILE': path,
-                        'MAWS_PROMPT': self.profile_name})
+                        "AWS_PROFILE": self.profile_name,
+                        "AWS_SHARED_CREDENTIALS_FILE": path,
+                        "MAWS_PROMPT": self.profile_name})
                     output_map.update({
                         x: None for x in ENV_VARIABLE_NAME_MAP.values()})
             elif self.output == "awscli":
@@ -435,25 +442,44 @@ class Login:
                                              self.credentials):
                     if self.profile_name != "default":
                         output_map.update({
-                            'AWS_PROFILE': self.profile_name,
-                            'AWS_SHARED_CREDENTIALS_FILE': None,
-                            'MAWS_PROMPT': self.profile_name
+                            "AWS_PROFILE": self.profile_name,
+                            "AWS_SHARED_CREDENTIALS_FILE": None,
+                            "MAWS_PROMPT": self.profile_name
                         })
                         output_map.update({
                             x: None for x in ENV_VARIABLE_NAME_MAP.values()})
                 else:
-                    logger.error('Unable to write credentials with aws-cli.')
+                    logger.error("Unable to write credentials with aws-cli.")
+            elif self.output == "boto":
+                # this output can be used directly by boto3
+                print(json.dumps({
+                        "aws_access_key_id": self.credentials["AccessKeyId"],
+                        "aws_secret_access_key": self.credentials["SecretAccessKey"],
+                        "aws_session_token": self.credentials["SessionToken"]},
+                    indent=2))
+                self.print_output_map = False
+            elif self.output == "js":
+                # this output can be used directly by the AWS Javascript SDK
+                print(json.dumps({
+                        "accessKeyId": self.credentials["AccessKeyId"],
+                        "secretAccessKey": self.credentials["SecretAccessKey"],
+                        "sessionToken": self.credentials["SessionToken"]},
+                    indent=2))
+                self.print_output_map = False
             else:
                 raise ValueError(
-                    'Output setting unknown : {}'.format(self.output))
+                    "Output setting unknown : {}".format(self.output))
+
+            if 'ExpirationSeconds' in self.credentials:
+                output_map['AWS_SESSION_EXPIRATION'] = self.credentials['ExpirationSeconds']
 
             message = "Environment variables set for role {}".format(
                 self.role_arn) if self.print_role_arn else None
 
-            if output_map:
+            if output_map and self.print_output_map:
                 print(output_set_env_vars(output_map, message))
 
-            if self.web_console:
+            if self.web_console or self.print_url:
                 self.aws_federate()
             else:
                 self.state = "finished"
@@ -473,7 +499,7 @@ class Login:
         })
         logger.debug("Web Console params: {}".format(query))
 
-        url_tuple = urlparse('https://signin.aws.amazon.com/federation')
+        url_tuple = urlparse("https://signin.aws.amazon.com/federation")
         url = urlunparse(url_tuple._replace(query=query))
         try:
             token = requests.get(url).json()
@@ -485,10 +511,10 @@ class Login:
         account_id = self.role_arn.split(":")[4]
         account_alias = self.role_map.get("aliases", {}).get(
             account_id, [account_id])[0]
-        role = self.role_arn.split(':')[5].split('/')[-1]
+        role = self.role_arn.split(":")[5].split("/")[-1]
         issuer_url_query = urlencode({"account": account_alias, "role": role})
         issuer_url = urlunparse(
-            ('https', self.issuer_domain, '/', '', issuer_url_query, ''))
+            ("https", self.issuer_domain, "/", "", issuer_url_query, ""))
 
         query = urlencode({
             "Action": "login",
@@ -500,13 +526,17 @@ class Login:
 
         logger.debug("Web browser console URL: {}".format(url))
 
-        if self.opened_tab:
+        if self.print_url:
+            print(url)
+            self.state = "finished"
+        elif self.opened_tab:
             self.state = "aws_federate"
             self.web_state["awsFederationUrl"] = url
         else:
             self.opened_tab = True
             webbrowser.open_new_tab(url)
             self.state = "finished"
+
         return url
 
 
